// debug level: 0-2 (0:disable, 1:break on error, 2:break at beginning)
$.level = 0;

//=================================================================
// Globals
//=================================================================

// UI strings to be localized
var uiVersion = "1.1"
var uiCopyright = "(c) Erik Krause 2004-2005";
var uiTitleLayersToFiles = "Export Layers with Alpha";
var uiButtonRun = "Run"; // never use the value "Ok"!
var uiButtonInfo = "Info";
var uiButtonCancel = "Cancel";
var uiButtonOk = "Ok";
var uiHelpText = "Please specify the format and location for saving each layer as a file.";
var uiInfoText = uiTitleLayersToFiles+" v"+uiVersion+" "+uiCopyright+"\n\n"+ 
                 "Exports one TIFF file with alpha channel from each layer that contains "+
                 "an image and that is marked visible in the source image.\n\n"+
                 "If the layer has a mask, this mask is used as alpha channel. Otherwise an "+
                 "empty (all white) alpha channel is created.\n\n"+
                 "Result images are numbered 0000, 0001, 0002 etc. starting from bottom layer "+
                 "to top. Check the 'Layer Names' box if you want to add layer names to the file "+
                 "names.\n\n"+
                 "See readme file for more information.\n\n"+
                 "If you have suggestions, comments or questions feel free to email me:\n"+
                 "erik.krause@gmx.de";
var uiLabelDestination = "Destination:";
var uiButtonBrowse = "Browse...";
var uiLabelFileNamePrefix = "File Name Prefix:";
var uiCheckbox16to8Bit = "reduce 16 to 8 Bit";
var uiCheckboxLayerNames = "Layer Names";
var uiLabelImageCompression = "Image Compression (TIFF):";
var uiRadiobuttonNone = "None";
var uiAlertSpecifyDestination = "Please specify destination.";
var uiAlertDestinationNotExist = "Destination does not exist.";
var uiTitleSelectDestination = "Select Destination";
var uiAlertDocumentMustBeOpened = "You must have a document open to export!";
var uiAlertWasSuccessful = " was successful.";
var uiAlertFaild = " failed.";

//=================================================================
// Functions
//=================================================================

// It carries dialog setting.
function initExportInfo(exportInfo)
{
    exportInfo.destination = new String("D:/IMAGES");
    exportInfo.fileNamePrefix = new String("untitled_");
    exportInfo.r16to8Bit = false;
    exportInfo.fileType = 3; // JPEG=1, PSD=2, TIFF=3, PDF=4, Targa=5, BMP=6
    exportInfo.icc = false;
    exportInfo.LNames = false;
    //exportInfo.jpegQuality = 8;
    exportInfo.tiffCompression = TIFFEncoding.TIFFLZW;
    //exportInfo.tiffJpegQuality = 8;
    //exportInfo.pdfEncoding = PDFEncoding.JPEG;
    //exportInfo.pdfJpegQuality = 8;

    try {
        exportInfo.destination = app.activeDocument.fullName.parent; // destination folder
        var tmp = app.activeDocument.fullName.name;
        exportInfo.fileNamePrefix = decodeURI(tmp.substring(0, tmp.indexOf("."))); // filename body part
    } catch(someError) {
        exportInfo.destination = new String("");
        exportInfo.fileNamePrefix = app.activeDocument.name; // filename body part
    }
}

// main dialog
function settingDialog(exportInfo)
{
    var dlgW = 390; // dialog width
    var dlgH = 285; // dialog height
    var btnW = 80;  // botton width
    var btnH = 20;  // botton height
    var txtW = 270; // text width
    var txtH = 20;  // text height
    var mgn  = 10;  // margin
    var pnlW = txtW;// panel width
    var pnlH = 60; // panel height
    var rb1W = 70;  // radio botton width level1
    var rb2W = 55;  // radio botton width level2
    var rbH = 20;   // radio botton height
    
    var bounds;
    var yLayout = mgn+txtH; // scanning y layout

    bounds = {x:0, y:0, width:dlgW, height:dlgH};
    var w = new Window("dialog", uiTitleLayersToFiles+" v"+uiVersion+" "+uiCopyright, bounds );

    bounds = {x:dlgW-btnW-mgn, y:yLayout, width:btnW, height:btnH};
    w.btnRun = w.add("button", bounds, uiButtonRun);
    w.btnRun.onClick = btnRunOnClick;

    yLayout += btnH + mgn;
    bounds = {x:dlgW-btnW-mgn, y:yLayout, width:btnW, height:btnH};
    w.btnInfo = w.add("button", bounds, uiButtonInfo, {name:'info'});
    w.btnInfo.onClick = function() {infoDialog(uiButtonInfo, uiInfoText);};

    yLayout += btnH + mgn;
    bounds = {x:dlgW-btnW-mgn, y:yLayout, width:btnW, height:btnH};
    w.btnCancel = w.add("button", bounds, uiButtonCancel, {name:'cancel'});
    w.btnCancel.onClick = function() { this.parent.close(0); };

    yLayout = mgn;  // reset

    bounds = {x:mgn, y:yLayout, width:180, height:txtH}; // if the bounds is too wide, "browse" button does not work.
    w.add("statictext", bounds, uiLabelDestination);

    yLayout += txtH;
    bounds = {x:mgn, y:yLayout, width:txtW-btnW-mgn-5, height:txtH}; // 5px smaller
    w.etDestination   = w.add("edittext", bounds, exportInfo.destination.toString());

    bounds = {x:mgn+txtW-btnW-5, y:yLayout, width:btnW+5, height:btnH}; // 5px wider
    w.btnBrowse= w.add("button", bounds, uiButtonBrowse);
    w.btnBrowse.onClick = btnBrowseOnClick;

    yLayout += txtH + mgn;
    bounds = {x:mgn, y:yLayout, width:txtW, height:txtH};
    w.add("statictext", bounds, uiLabelFileNamePrefix);

    yLayout += txtH;
    bounds = {x:mgn, y:yLayout, width:txtW, height:txtH};
    w.etFileNamePrefix = w.add("edittext", bounds, exportInfo.fileNamePrefix.toString());

    yLayout += txtH + mgn;
    bounds = {x:mgn, y:yLayout, width:txtW/2, height:txtH};
    w.LNames = w.add( "checkbox", bounds, uiCheckboxLayerNames);
    w.LNames.value = exportInfo.LNames;
    bounds = {x:mgn+txtW/2, y:yLayout, width:txtW/2, height:txtH};
    w.cb16to8 = w.add( "checkbox", bounds, uiCheckbox16to8Bit);
    w.cb16to8.value = exportInfo.r16to8Bit;

    yLayout += txtH + mgn;
    bounds = {x:mgn, y:yLayout, width:pnlW, height:pnlH};
    w.pnlFT = w.add( "panel", bounds, uiLabelImageCompression );

    bounds = {x:mgn, y:mgn+txtH, width:rb2W, height:rbH};
    w.rbNone = w.pnlFT.add( "radiobutton", bounds, uiRadiobuttonNone);
    bounds = {x:mgn*2+rb2W, y:mgn+txtH, width:rb2W, height:rbH};
    w.rbLzw = w.pnlFT.add( "radiobutton", bounds, "LZW");
    bounds = {x:mgn*2+rb2W*2, y:mgn+txtH, width:rb2W, height:rbH};
    w.rbZip = w.pnlFT.add( "radiobutton", bounds, "ZIP");


    bounds = {left:mgn, top:dlgH-mgn-50, right:dlgW-mgn, bottom:dlgH-mgn};
    w.pnlHelp = w.add( "panel", bounds, "" ); 
    bounds = {x:mgn, y:mgn, width:dlgW-mgn*4, height:30};
    w.pnlHelp.add("statictext", bounds, uiHelpText, {multiline:true});

    switch (exportInfo.tiffCompression) {
        case TIFFEncoding.NONE:     w.rbNone.value = true; break;
        case TIFFEncoding.TIFFLZW:  w.rbLzw.value  = true; break;
        case TIFFEncoding.TIFFZIP:  w.rbZip.value  = true; break;
        default: w.rbNone.value = true;    break;
    }
  
    w.center(); // centering dialog
    var result = w.show();
    if ( 0 == result)   return result;  // close to quit
    
    // get setting from dialog
    exportInfo.destination = w.etDestination.text;
    exportInfo.fileNamePrefix = w.etFileNamePrefix.text;
    exportInfo.r16to8Bit = w.cb16to8.value;
    exportInfo.LNames = w.LNames.value;
    exportInfo.fileType = 3; //TIFF
    //exportInfo.icc = false;
    exportInfo.tiffCompression = TIFFEncoding.NONE;
    if (w.rbNone.value) exportInfo.tiffCompression = TIFFEncoding.NONE;
    if (w.rbLzw.value)  exportInfo.tiffCompression = TIFFEncoding.TIFFLZW;
    if (w.rbZip.value)  exportInfo.tiffCompression = TIFFEncoding.TIFFZIP;

    return result;
}


function btnRunOnClick()    // call back function
{
    // check if the setting is properly
    var destination = this.parent.etDestination.text;
    if (destination.length == 0) {
        alert(uiAlertSpecifyDestination);
        return;
    }
    var testFolder = new Folder(destination);
    if (!testFolder.exists) {
        alert(uiAlertDestinationNotExist);
        return;
    }
    
    this.parent.close(1);
}

function btnBrowseOnClick() // call back function
{
    var defaultFolder = this.parent.etDestination.text;
    var testFolder = new Folder(this.parent.etDestination.text);
    if (!testFolder.exists) defaultFolder = "~";
    var selFolder = Folder.selectDialog(uiTitleSelectDestination, defaultFolder);
    if ( selFolder != null ) {
        this.parent.etDestination.text = selFolder.toString();
    }
    return;
}

// info dialog
function infoDialog(Title, Info)
{
    var dlgW = 370; // dialog width
    var dlgH = 335; // dialog height
    var btnW = 80;  // botton width
    var btnH = 20;  // botton height
    var mgn  = 10;  // margin

    var bounds;

    bounds = {x:0, y:0, width:dlgW, height:dlgH};
    var w = new Window("dialog", Title, bounds );

    bounds = {x:mgn, y:mgn, width:dlgW-mgn*2, height:dlgH-mgn*3-btnH};
    w.infoText = w.add("statictext", bounds, Info, {multiline:true});

    bounds = {x:dlgW/2-btnW/2, y:dlgH-mgn-btnH, width:btnW, height:btnH};
    w.btnOk = w.add("button", bounds, uiButtonOk, {name:'ok'});
    w.btnOk.onClick = function() { this.parent.close(1); };

    w.center(); // centering dialog
    w.show();
}

function DuplicateWithMerge(DocName) 
{
var id30 = charIDToTypeID( "Dplc" );
    var desc7 = new ActionDescriptor();
    var id31 = charIDToTypeID( "null" );
        var ref4 = new ActionReference();
        var id32 = charIDToTypeID( "Dcmn" );
        var id33 = charIDToTypeID( "Ordn" );
        var id34 = charIDToTypeID( "Frst" );
        ref4.putEnumerated( id32, id33, id34 );
    desc7.putReference( id31, ref4 );
    var id35 = charIDToTypeID( "Nm  " );
    desc7.putString( id35, DocName );  
    var id36 = charIDToTypeID( "Mrgd" );
    desc7.putBoolean( id36, true );
executeAction( id30, desc7, DialogModes.NO );
} //...Duplicate with merge

// =======================================================
function SelectTransparency() 
{
var id37 = charIDToTypeID( "setd" );
    var desc8 = new ActionDescriptor();
    var id38 = charIDToTypeID( "null" );
        var ref5 = new ActionReference();
        var id39 = charIDToTypeID( "Chnl" );
        var id40 = charIDToTypeID( "fsel" );
        ref5.putProperty( id39, id40 );
    desc8.putReference( id38, ref5 );
    var id41 = charIDToTypeID( "T   " );
        var ref6 = new ActionReference();
        var id42 = charIDToTypeID( "Chnl" );
        var id43 = charIDToTypeID( "Chnl" );
        var id44 = charIDToTypeID( "Trsp" );
        ref6.putEnumerated( id42, id43, id44 );
    desc8.putReference( id41, ref6 );
executeAction( id37, desc8, DialogModes.NO );

} //...Set Selection to transparency

// =======================================================
function SaveSelectionToChannel(ChName)
{
var id45 = charIDToTypeID( "Dplc" );
    var desc9 = new ActionDescriptor();
    var id46 = charIDToTypeID( "null" );
        var ref7 = new ActionReference();
        var id47 = charIDToTypeID( "Chnl" );
        var id48 = charIDToTypeID( "fsel" );
        ref7.putProperty( id47, id48 );
    desc9.putReference( id46, ref7 );
    var id49 = charIDToTypeID( "Nm  " );
    desc9.putString( id49, ChName );
executeAction( id45, desc9, DialogModes.NO );
}//...Save selection as Alpha 1

// =======================================================
function FlattenToBackground()
{
var id50 = charIDToTypeID( "FltI" );
executeAction( id50, undefined, DialogModes.NO );

}//...Flatten to Background

// =======================================================
function ConvertBitDepth(BitDepth) // 8 or 16
{
var id51 = charIDToTypeID( "CnvM" );
    var desc10 = new ActionDescriptor();
    var id52 = charIDToTypeID( "Dpth" );
    desc10.putInteger( id52, BitDepth );
executeAction( id51, desc10, DialogModes.NO );
}//...convert Bit Depth

// =======================================================
function SelectLayerSet(SetName)
{
var id586 = charIDToTypeID( "slct" );
    var desc156 = new ActionDescriptor();
    var id587 = charIDToTypeID( "null" );
        var ref65 = new ActionReference();
        var id588 = charIDToTypeID( "Lyr " );
        ref65.putName( id588, SetName);
    desc156.putReference( id587, ref65 );
executeAction( id586, desc156, DialogModes.NO );
}//select layer set

// =======================================================
function DeleteLayerSet()
{
var id589 = charIDToTypeID( "Dlt " );
    var desc157 = new ActionDescriptor();
    var id590 = charIDToTypeID( "null" );
        var ref66 = new ActionReference();
        var id591 = charIDToTypeID( "Lyr " );
        var id592 = charIDToTypeID( "Ordn" );
        var id593 = charIDToTypeID( "Trgt" );
        ref66.putEnumerated( id591, id592, id593 );
    desc157.putReference( id590, ref66 );
    var id594 = stringIDToTypeID( "deleteContained" );
    desc157.putBoolean( id594, false );
executeAction( id589, desc157, DialogModes.NO );
}// delete without content

function zeroSuppress (num, digit)
{
    var tmp = num.toString();
    while(tmp.length < digit)   tmp = "0" + tmp;
    return tmp
}

function removeAllLayerSets(obj)   // obj can be DocRef or LayerSet
{
    for( var i = obj.layerSets.length-1; 0 <= i; i--) {
      removeAllLayerSets(obj.layerSets[i]);  // recursive call for nested layer sets.
      SelectLayerSet(obj.layerSets[i].name); // select set by name
      DeleteLayerSet();                      // delete it but not containing layers 
    }
}

function drawselectbox(x1,y1,x2,y2){
var selregion = [[x1,y1],[x2,y1],[x2,y2],[x1,y2],[x1,y1]];

try{app.activeDocument.selection.select(selregion,SelectionType.REPLACE)}
catch(e){};

selregion=null;
}


function main()
{
//  $.bp();
  if ( app.documents.length <= 0 ) {
      alert( uiAlertDocumentMustBeOpened );
      return;
  }
  try {
    var exportInfo = new Object(); 
    drawselectbox(0,0,2,2);
    initExportInfo(exportInfo); 
    if (0 == settingDialog(exportInfo)) return; // quit

    var docName = app.activeDocument.name;  // save the app.activeDocument name before duplicate.
    var layerVisibility = new Array();  // save layer visibility
    app.activeDocument = app.documents[docName]; 
    var obj = app.activeDocument 
    var duplicated = obj.layerSets.length > 0; // if there are layer sets we must duplicate.
    if (duplicated) 
    { // layer sets must be removed leaving the contens intact...
      obj = obj.duplicate();
      removeAllLayerSets(obj);
    }  
    var layerNumber = obj.artLayers.length-1
    var layerCounter = 0;
    // from bottom up process layers.
    for( var i = layerNumber; i >= 0; i--) 
    { // first make all image layers invisible.
      layerVisibility[i] = obj.artLayers[i].visible; // save visibility
      if (obj.artLayers[i].kind == "LayerKind.NORMAL") // not for adjustment layers etc.
        obj.artLayers[i].visible = false;              // make invisible. 
    }
    try {
        // again, from bottom up
        for( var i = layerNumber; i >= 0; i--) 
          if ((layerVisibility[i] == true) && (obj.artLayers[i].kind == "LayerKind.NORMAL"))
          { // process image layers only
            obj.artLayers[i].visible = true; // set layer visible
            DuplicateWithMerge(obj.artLayers[i].name); // duplicate with merge, set layer nam as image name
            var dup = app.activeDocument; // get active (new) document
            dup.artLayers[0].isBackgroundLayer = false; // if Background, create layer 0 instead
            SelectTransparency(); // load selection
            try {
              for (var ci = 0; ci < dup.channels.length; ci++) {
                if (dup.channels[ci].kind != ChannelType.COMPONENT) 
                  dup.channels[ci].remove();
              }
            } catch(el){};  
            SaveSelectionToChannel("Alpha 1"); // save selection to channel
            FlattenToBackground(); // flatten image (was layered previously).
            // reduce to 8 bit if requested 
            if (exportInfo.r16to8Bit && dup.bitsPerChannel == BitsPerChannelType.SIXTEEN) 
              ConvertBitDepth(8);  // current versions of enblend don't support 16 bit
            // create filename
            var fileNameBody = exportInfo.fileNamePrefix;
            //fileNameBody += "_" + zeroSuppress(layerNumber-i, 4);
            fileNameBody += "_" + zeroSuppress(layerCounter, 4);
            if (exportInfo.LNames)
              fileNameBody += "_" + dup.name;
            fileNameBody  = fileNameBody.replace(/\//g, "_");  // '/' -> '_'
            if (fileNameBody.length > 120) fileNameBody = fileNameBody.subString(0,120);
            // save file
            var saveFile = new File(exportInfo.destination + "/" + fileNameBody + ".tif");
            tiffSaveOptions = new TiffSaveOptions();
            tiffSaveOptions.embedColorProfile = exportInfo.icc;
            tiffSaveOptions.imageCompression = exportInfo.tiffCompression;
            dup.saveAs(saveFile, tiffSaveOptions, true, Extension.LOWERCASE);
            dup.close(SaveOptions.DONOTSAVECHANGES);
            // set original document active
            app.activeDocument = obj;
            obj.artLayers[i].visible = false; // disable current layer.
            layerCounter++ // increment layer number
          } // main loop.
        if (duplicated) // is duplicate?
          obj.close(SaveOptions.DONOTSAVECHANGES); // then it isn't needed any more
        else  // if not...
          for( var i = obj.artLayers.length-1; i >= 0; i--) // set original visibility
            obj.artLayers[i].visible = layerVisibility[i];
        alert(uiTitleLayersToFiles + uiAlertWasSuccessful);
            
    } catch (e1) {
        alert(e1);
        if (app.activeDocument == dup) {// close document if open.
          dup.close(SaveOptions.DONOTSAVECHANGES);
          // set original document active
          app.activeDocument = obj;
        }  
        if (duplicated) // is duplicate?
          obj.close(SaveOptions.DONOTSAVECHANGES); // then it isn't needed any more
        else  // if not...
          for( var i = obj.artLayers.length-1; i >= 0; i--) // set original visibility
            obj.artLayers[i].visible = layerVisibility[i];
    }
  } catch(e2) {
      alert(e2);
  }    
}


//===============================================================
// Dispatch
//===============================================================

main();
